'use server';

import { redirect } from 'next/navigation';
import fs from 'fs/promises';
import sharp from 'sharp';
import { Resend } from 'resend';
import path from 'path';
import crypto from 'crypto';

import { env } from '~/env';

import { FormSchema } from '../schemas';
import FormEmail from '../email/FormEmail';

const resend = new Resend(env.RESEND_API_KEY);

export async function dentalFormSubmit(values: FormData) {
  const validatedFields = FormSchema.safeParse({
    ...Object.fromEntries(values.entries()),
    contattato: (values.get('contattato') as string).split(','),
  });

  if (!validatedFields.success) {
    throw new Error('Campi non validi!');
  }

  const {
    parodontite,
    anni,
    dentistaVisitaItalia,
    panaromicaDentale,
    panoramica,
    nome,
    email,
    telefono,
    tuaDomanda,
    contattato,
  } = validatedFields.data;

  let imagePath: string | undefined;
  if (panoramica) {
    if (!panoramica.type.startsWith('image/')) {
      throw new Error("Questo file non è un'immagine!");
    }

    // Define the upload directory outside of the public folder
    const uploadDir = path.join(process.cwd(), 'uploads', 'panoramica');
    await fs.mkdir(uploadDir, { recursive: true });

    const uniqueFileName = `${nome}-${crypto.randomUUID()}.webp`;
    imagePath = `/api/uploads/panoramica/${uniqueFileName}`;

    try {
      const imageBuffer = Buffer.from(await panoramica.arrayBuffer());
      const webpBuffer = await sharp(imageBuffer).webp().toBuffer();

      await fs.writeFile(path.join(uploadDir, uniqueFileName), webpBuffer);
    } catch {
      throw new Error("Si è verificato un errore con l'immagine!");
    }
  }

  const submittedData = {
    parodontite,
    anni,
    dentistaVisitaItalia,
    panaromicaDentale,
    panoramica,
    nome,
    email,
    telefono,
    tuaDomanda,
    contattato,
  };

  const data = await resend.emails.send({
    from: env.SENDER_EMAIL,
    to: env.SEND_TO_EMAIL,
    subject: 'Nuovo Modulo',
    react: <FormEmail data={submittedData} imagePath={imagePath} />,
  });

  if (data.error) {
    throw new Error('È successo qualcosa di inaspettato!');
  }

  redirect('https://primodentalclinic.com/');
}
